using System;
using System.Collections.Generic;
using System.Security.Cryptography.X509Certificates;
using System.Threading;
using System.Threading.Tasks;

namespace Ascon.Pilot.SDK.XpsDigitalSigner
{
    public class BatchSigner
    {
        private readonly List<IDataObject> _selectedObjects;
        private readonly X509Certificate2 _certificate;
        private readonly IXpsDigitalSigner _digitalSigner;
        private readonly IEnumerable<int> _suitablePositions;
        private Semaphore _semaphorePool;
        private Guid _currentObjectId;

        public BatchSigner(List<IDataObject> selectedObjects, IEnumerable<int> positions, X509Certificate2 certificate, IXpsDigitalSigner digitalSigner)
        {
            _suitablePositions = positions;
            _selectedObjects = selectedObjects;
            _certificate = certificate;
            _digitalSigner = digitalSigner;
        }

        public void Start()
        {
            _semaphorePool = new Semaphore(0, 1);
            foreach (var doc in _selectedObjects)
            {
                Task.Factory.StartNew(() =>
                {
                    _semaphorePool.WaitOne();
                    _currentObjectId = doc.Id;
                    _digitalSigner.SignDocumentWithCertificate(_currentObjectId, _suitablePositions, _certificate);
                });
            }
            _semaphorePool.Release();
        }

        public int Count => _selectedObjects.Count;

        public void Next()
        {
            _semaphorePool.Release();
        }

        public Guid Current()
        {
            return _currentObjectId;
        }
    }
}