﻿using System.Windows.Input;
using System.Windows.Media;
using Ascon.Pilot.Theme.Controls;
using ChangeFilesMonitor.DelegateCommand;

namespace ChangeFilesMonitor
{
    public class ToolbarButtonItem : PropertyChangedBase, IToolbarButtonItem
    {
        private string _name;
        private object _header;
        private ImageSource _icon;
        private ICommand _command;
        private object _commandParameter;
        private bool? _isEnabled = true;
        private bool _showCaption = true;

        public ToolbarButtonItem(string name)
        {
            Name = name;
        }

        public object Header
        {
            get { return _header; }
            set
            {
                _header = value;
                NotifyOfPropertyChanged(nameof(Header));
            }
        }

        public ImageSource Icon
        {
            get { return _icon; }
            set
            {
                _icon = value;
                NotifyOfPropertyChanged(nameof(Icon));
            }
        }

        public ICommand Command
        {
            get { return _command; }
            set
            {
                _command = value;
                NotifyOfPropertyChanged(nameof(Command));
            }
        }

        public object CommandParameter
        {
            get { return _commandParameter; }
            set
            {
                _commandParameter = value;
                NotifyOfPropertyChanged(nameof(CommandParameter));
                var canExecute = Command as DelegateCommandBase;
                if (canExecute != null)
                    canExecute.RaiseCanExecuteChanged();
            }
        }

        public bool? IsEnabled
        {
            get { return _isEnabled; }
            private set
            {
                _isEnabled = value;
                NotifyOfPropertyChanged(nameof(IsEnabled));
            }
        }

        public bool ShowCaption
        {
            get { return _showCaption; }
            private set
            {
                _showCaption = value;
                NotifyOfPropertyChanged(nameof(ShowCaption));
            }
        }

        public string Name
        {
            get { return _name; }
            set
            {
                _name = value;
                NotifyOfPropertyChanged(nameof(Name));
            }
        }
    }
}
