﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Policy;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Ascon.Pilot.SDK;
using Ascon.Pilot.SDK.Extensions;
using ProjectCloneWizard.Common;
using ProjectCloneWizard.Data;
using ProjectCloneWizard.ObjectsTree;

namespace ProjectCloneWizard.ViewModels
{
    class UpdateStructProcessPageViewModel : ProcessPageViewModel
    {
        public UpdateStructProcessPageViewModel(CreationInfo creationInfo, IObjectModifier modifier, IObjectsRepository repository, IFileProvider fileProvider) 
            : base(creationInfo, modifier, repository, fileProvider)
        {
        }

        protected override async Task<bool> Process(IEnumerable<IDataObject> templateObjects)
        {
            Progress = "Загрузка всех детей выбранного элемента";
            var children = await _repository.AsyncMethods().GetObjectsAsync(_creationInfo.Parent.Children, (o) => o, CancellationToken.None);
            var projectChildrenTitles = children.Select(GetObjectTitle).ToList();

            return await Task<bool>.Factory.StartNew(() => 
            {
                //Create project
                _idMap.Add(_creationInfo.Template.Id, _creationInfo.Parent.Id);

                var objectList = new List<NodeViewModel>();
                FillNodeList(_creationInfo.ObjectsNodes, objectList);

                var storageList = new List<NodeViewModel>();
                FillNodeList(_creationInfo.StorageNodes, storageList);

                var result = false;
                foreach (var dataObject in templateObjects)
                {
                    _cts.Token.ThrowIfCancellationRequested();

                    if (dataObject.Id == _creationInfo.Template.Id)
                        continue;

                    var templateTitle = GetObjectTitle(dataObject);
                    if (projectChildrenTitles.Contains(templateTitle))
                        continue;

                    //если есть в списке и установлен IsChecked создаем
                    var node = objectList.FirstOrDefault(n => n.Id == dataObject.Id);
                    if (node != null && node.IsChecked)
                        result = CreateObjectClone(dataObject, _creationInfo.CopyAccessForObjects);

                    var storageNode = storageList.FirstOrDefault(n => n.Id == dataObject.Id);
                    if (storageNode != null && storageNode.IsChecked)
                    {
                        if (storageNode.Type.IsEcmDocument())
                            continue;

                        result = CreateObjectClone(dataObject, _creationInfo.CopyAccessForFiles);
                    }

                }
                return result;
            }, _cts.Token);
        }

        private string GetObjectTitle(IDataObject dataObject)
        {
            var type = dataObject.Type;
            if (dataObject.Attributes.ContainsKey(SystemAttributeNames.SMART_FOLDER_TITLE))
            {
                return dataObject.Attributes[SystemAttributeNames.SMART_FOLDER_TITLE].ToString();
            }

            if (dataObject.Id == SystemObjectIds.RootObjectId)
                return type.Title;

            var sb = new StringBuilder();
            foreach (var attr in type.Attributes)
            {
                var attributeText = GetAttributeText(dataObject, attr);
                if (sb.Length != 0 && !string.IsNullOrEmpty(attributeText))
                {
                    sb.Append(" - ");
                }
                sb.Append(attributeText);
            }
            return sb.ToString();
        }

        private string GetAttributeText(IDataObject dataObject, IAttribute attr)
        {
            if (attr.Type == AttributeType.Numerator)
            {
                return string.Empty;
            }

            object value;
            dataObject.Attributes.TryGetValue(attr.Name, out value);
            var strValue = value?.ToString();

            if (attr.Name == SystemAttributeNames.PROJECT_ITEM_NAME)
                return strValue;

            if (attr.ShowInObjectsExplorer)
                return strValue;

            return string.Empty;
        }
    }
}
