﻿using System;
using System.IO;
using System.Linq;
using System.Reflection;
using System.ServiceProcess;
using Newtonsoft.Json;
using Pilot.Xps.Connection.Server;

namespace Pilot.Xps.Console
{
    class Program
    {
        public static string ServiceName = "Pilot.Xps.Service";
        private static XpsServiceContext _context;
        private static XpsServiceContextLogger _contextLogger;

        static void Main(string[] args)
        {
            if (!Environment.UserInteractive)
                // running as service
                using (var service = new Service())
                    ServiceBase.Run(service);
            else
            {
                if (args.Contains("-i") || args.Contains("--install"))
                {
                    var filename = Assembly.GetExecutingAssembly().Location;
                    ServiceInstaller.InstallAndStart(ServiceName, ServiceName, filename);
                    System.Console.WriteLine("Service installed successfully");
                    return;
                }

                if (args.Contains("-u") || args.Contains("--uninstall"))
                {
                    ServiceInstaller.Uninstall(ServiceName);
                    System.Console.WriteLine("Service uninstalled successfully");
                    return;
                }

                // running as console app
                System.Console.WriteLine("Connecting...");
                Start(args);

                System.Console.WriteLine("Connected");
                System.Console.WriteLine("Press any key to stop...");
                System.Console.ReadKey(true);

                Stop();
            }
        }

        public static void Start(string[] args)
        {
            var settings = LoadAppSettings();
            _context = new XpsServiceContext(settings);
            _contextLogger = new XpsServiceContextLogger(_context);
            _contextLogger.Start();
        }

        public static void Stop()
        {
            _context.Dispose();
        }

        private static AppSettings LoadAppSettings()
        {
            AppSettingsJson json;
            using (var reader = new StreamReader(AppDomain.CurrentDomain.BaseDirectory + "/appsettings.json"))
            {
                json = JsonConvert.DeserializeObject<AppSettingsJson>(reader.ReadToEnd());
            }

            var settings = new AppSettings(json);
            return settings;
        }
    }
}
